#ifndef RCSID_DEFINED
static char RCSID[] = "$Id: client.c 1.30 1997/04/04 19:31:02 ken Development $ ELAN";
#define RCSID_DEFINED
#endif

/*
 ****************************************************************************
 * Copyright (c) 1989-1996 Elan Computer Group, Inc.
 * All Rights Reserved.
 * -------------------------------------------------------------------------
 * client.c - A sample Elan License Manager application.
 *
 * This example has everything including the kitchen sink. For hot shots only
 * that want to get heavy into Elan License Manager fringe funtions.
 *
 * If you're new to Elan License Manager, check out:
 *
 *	ezclient.c	Example using our easy two function API.  It's easy!
 *
 *	simple.c	An example using our simple (standard) API.
 *
 ****************************************************************************
 */

#include <stdio.h>
#include <ctype.h>
#include <signal.h>

#if defined (NETWARE) && !defined(_WINDOWS)
	/* UNLICENSED CODE REMOVED */
#endif

#ifdef _WINDOWS
	/* UNLICENSED CODE REMOVED */
#endif

#include "slm.h"

#if defined (ANSI_C)
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#define TRUE	1
#define FALSE	0
#define plural(n)	((n) == 1 ? "":"s")

#define DEFAULT_FEATURE		"99"
#define EXPIRATION_WARN		30 /* Warn when 30 days within expiration */

/*
 * The heartbeat in a typical application is about every 60 seconds.
 * We're using 10 seconds here just so that you can seen things happen 
 * a lot faster.  Use the -b option or change this define if you would 
 * like to change the heartbeat time.
 */
static unsigned int beat = 10;

static char *progname;
static long unsigned int pid;
static char *Keydir = slm_keydir;
static char server[SLM_HOST_LEN+1];

int IsRunning;

SLM_INSTANCE instance;
SLM_CONNECTION connection;

extern char *ctime();
#ifndef atol
extern long atol();
#endif

#ifdef ANSI_C
void quit (char *fmt, ...) {
#else
void 
quit(va_alist)
va_dcl
{
#endif
	va_list args;
#ifdef ANSI_C
	va_start (args, fmt);
#else	
	char *fmt;

	va_start (args);
	fmt = va_arg(args, char *);
#endif	

	vfprintf (stderr, fmt, args);
	fprintf (stderr, "\n");

	va_end (args);

	slm_disconnect (instance, connection);
	slm_endapi (&instance);
	exit (-1);
}

/*
 * Routine called when process exits.
 *	1. Disconnect from license server (slm_disconnect).
 *	2. Exit with error code "code".
 */
#ifdef ANSI_C
void byebye (int code) {
#else 
void byebye(code)
int code;
{
#endif
	IsRunning = FALSE;
	printf ("\n[%ld] PID %lu disconnecting.\n", slm_clientid(instance, connection), pid);

	slm_disconnect (instance, connection);

	printf ("PID %lu exiting.\n", pid);

	slm_endapi (&instance);

	exit (code);
}

/*
 * Catch all normal signals (hangup, interrupt, quit, term)
 * and call "byebye" routine when any of these signals occur.
 */
void
catchem()
{
#ifdef SIGHUP
	signal (SIGHUP, byebye);
#endif
#ifdef SIGINT
	signal (SIGINT, byebye);
#endif
#ifdef SIGQUIT
	signal (SIGQUIT, byebye);
#endif
#ifdef SIGTERM
	signal (SIGTERM, byebye);
#endif
}

/*
 * Do all the initialization stuff:
 *	1. Catch signals.
 *	2. Connect with the licenser server (slm_connect).
 *	3. Exit if any problems.
 */
#ifdef ANSI_C
void do_init (char *feature) {
#else 
void
do_init (feature)
char *feature;
{
#endif
	extern char *getenv();

#if !defined(NETWARE) || (defined(NETWARE) && defined(_WINDOWS))
	catchem();			/* Catch signals */
#endif	

	server[0] = '\0';
	connection = slm_connect(instance, server, feature, Keydir, SLM_NOCALLBACK);

	if (connection < 0)
	{
		printf ("%s.\n", slm_message (instance, server, feature, connection));
		exit (1);
	}

	printf ("[%ld] Connected with server \"%s\".\n", slm_clientid(instance, connection), server);
}

#ifdef ANSI_C
void Exception ( long code, char *feature ) {
#else
void
Exception(code, feature)
long code;
char *feature;
{
#endif
	if (code == SLM_SRV_DOWN) /* non-fatal condition */
	printf ("License server down (not-fatal)\n");
	else {
		/* We just exit - You will want to do something fancier. */
		printf ("%s.\n", slm_message(instance, server, feature, code));
		byebye(1);
	}
}

#ifdef ANSI_C
void check_expiration ( char *feature, int pack_major, int pack_minor) {
#else
void
check_expiration (feature, pack_major, pack_minor)
char *feature;
int pack_major;
int pack_minor;
{
#endif
	struct slm_packinfo info;
	time_t now;
	int days;

	if (slm_getpackinfo (instance, connection, &info, feature, 0, (long )pack_major, (long )pack_minor) <= 0)
		return;

	if (info.expiration == 0)	/* No expiration */
	{
		printf ("The license has no expiration date.\n");
		return;
	}

	time (&now);
	days = (info.expiration - now) / SLM_DAY;

#if 0	/* Skip -- show expiration all the time in our example */
	/* UNLICENSED CODE REMOVED */
#endif

	if (days == 0)
		printf ("WARNING: The license (%s) will expire tonight!\n",
			info.alias[0] ? info.alias:info.feature);
	else
		printf ("WARNING: The license (%s) will expire in %d days.\n",
			info.alias[0] ? info.alias:info.feature, days);
}


#ifdef ANSI_C
void showinfo ( char *feature ) {
#else
void
showinfo(feature)
char *feature;
{
#endif
	struct slm_featureinfo info;
	struct slm_user user;
	char result[SLM_VENDORDATA_LEN+1];
	long i, cid;
	long freepool;


	cid = slm_clientid(instance, connection);	/* My Client ID */

	if (slm_getulist(instance, connection, &user, feature, cid) <= 0)
		return;

	slm_getfeatureinfo (instance, connection, &info, feature, 0);

	printf ("    Feature = \"%s\"\n", info.feature);

	if ((freepool = info.numlic - info.numresv) < 0)
		freepool = 0;
	printf ("    Total licenses  = %ld (%ld free pool + %ld reserved).\n", 
		info.numlic, freepool, info.numresv);
	if (info.numresv_for_me)
		printf ("    Licenses reserved for me = %ld\n",
				info.numresv_for_me);
	printf ("    Licenses in use = %ld (%ld remain)\n",
		info.inuse_resv + info.inuse_free,
		info.numlic - (info.inuse_resv + info.inuse_free));

	/* Warn if we're close */
	check_expiration(feature, user.packid_major, user.packid_minor);
}

#include <time.h>

typedef enum months {
   JAN=1,FEB=2,MAR=3,APR=4,MAY=5,JUN=6,JUL=7,AUG=8,SEP=9,OCT=10,NOV=11,DEC=12
} month_t;

/*
 * Convert Day-Month-Year to POSIX time.
 * Inputs:
 *	Day:	1-31
 *	Month:  enum month_t above or 1-12
 *	Year:	1970 and beyond.
 */
#ifdef ANSI_C
time_t dateToPosix(int day, month_t month, int year)
#else
time_t dateToPosix(day, month, year)
int day, year;
month_t month;
#endif
{
	struct tm tm;
	memset (&tm, 0, sizeof (tm));
	tm.tm_mday = day;		/* 1 - 31 */
	tm.tm_mon = month - 1;		/* 1 - 12 */
	tm.tm_year = year - 1900;	/* 1900- */
	return(mktime (&tm));		/* Convert tm to POSIX */
}

/*
 * The code below is meant as an example of 7 different 
 * constraints (numbered 0-6).  Default 0 gives you no constraints.
 */
#ifdef ANSI_C
void set_constraints(slm_constraints_t *constraints, int constraint_num)
#else
void set_constraints(constraints, constraint_num)
slm_constraints_t *constraints;
int constraint_num;
#endif
{
	slm_constraints_t *cp = constraints;

	switch (constraint_num)
	{
	case 0:				/* None */
		printf ("Constraint: none\n");
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	case 1:				/* release >= 30 */
		printf ("Constraint: release >= 30\n");
		cp->field = SLM_CONSTRAINT_VENDOR("release");
		cp->op = SLM_GE;
		strcpy (cp->arg, "30");
		cp++;
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	case 2:				/* Start date >= 1-May-95 */
		printf ("Constraint: Start date >= 1-May-95\n");
		cp->field = SLM_CONSTRAINT_START_DATE;
		cp->op = SLM_GE;
		sprintf (cp->arg, "%ld", dateToPosix(1, MAY, 1995));
		cp++;
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	case 3:				/* Expiration date < 1-May-99 */
		printf ("Constraint: Expiration date < 1-May-99\n");
		cp->field = SLM_CONSTRAINT_EXPIRATION;
		cp->op = SLM_LT;
		sprintf (cp->arg, "%ld", dateToPosix(1, MAY, 1999));
		cp++;
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	case 4:				/* Expiration date < 1-May-98 */
		printf ("Constraint: Expiration date < 1-May-98\n");
		cp->field = SLM_CONSTRAINT_EXPIRATION;
		cp->op = SLM_LT;
		sprintf (cp->arg, "%ld", dateToPosix(1, MAY, 1998));
		cp++;
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	case 5:				/* Domain is B */
		printf ("Constraint: Domain == B\n");
		cp->field = SLM_CONSTRAINT_DOMAIN;
		cp->op = SLM_EQ;
		strcpy (cp->arg, "B");
		cp++;
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	case 6:				/* Domain >= B (B or C) and release <= 25 */
		printf ("Constraint: Domain >= B and release <= 25\n");
		cp->field = SLM_CONSTRAINT_DOMAIN;
		cp->op = SLM_GE;
		strcpy (cp->arg, "B");
		cp++;
		cp->field = SLM_CONSTRAINT_VENDOR("release");
		cp->op = SLM_LE;
		strcpy (cp->arg, "25");
		cp++;
		cp->field = SLM_CONSTRAINT_EOL;
		break;
	default:
		printf ("No such constraint %d\n", constraint_num);
	}
}

#ifdef ANSI_C
void do_getlicense ( char *feature, unsigned long ntokens, unsigned long options, int constraint_num) {
#else
void
do_getlicense(feature, ntokens, options, constraint_num)
char *feature;
long options, ntokens;
int constraint_num;
{
#endif
	long x;
	long soft_exceeded;
	slm_constraints_t constraints[SLM_MAX_CONSTRAINTS+1];

	set_constraints (constraints, constraint_num);/* Set constraints, if you want any */

	/* 
	 * Try to get a license
	 */
	x = slm_getlicense(instance, connection, feature, options, ntokens, constraints, &soft_exceeded);

	if (x != SLM_OK)
	{
		if (x < 0)
			printf ("%s.\n", slm_message(instance, server, feature, x));
		if (options & SLM_QUEUE)
		{
			printf ("[%ld] No licenses available. Queueing...\n", slm_clientid(instance, connection));
			options = (options & ~SLM_QUEUE) | SLM_GETQUEUED;
			while (x != 0)
			{
				if (x >= 0)
					printf("[%ld] Number %d in the queue\n",
						slm_clientid(instance, connection), x);
				else
					printf ("%s.\n",
					    slm_message(instance, server, feature, x));
#if !defined (WINDOWS_31)
				slu_sleep (15);
#endif
				x = slm_getlicense(instance, connection, feature, options, ntokens, NULL, &soft_exceeded);
			} 
		}
		else
			byebye(1);
	}

	printf ("[%ld] %sLicense for \"%s\" granted: %ld token%s.\n",
		slm_clientid(instance, connection), 
		options & SLM_SHARED ? "Shared ":"",
		feature,
		ntokens, 
		plural(ntokens));

	if (soft_exceeded != 0)
	{
		printf ("***********************************************\n");
		printf ("* Soft license limit exceeded by %ld license(s)\n",
					soft_exceeded);
		printf ("* Please acquire additional licenses.         *\n");
		printf ("***********************************************\n");
	}

	showinfo (feature);
	printf ("\n");
}

#ifdef ANSI_C
void sethold ( long holdval ) {
#else
void
sethold(holdval)
long holdval;
{
#endif
	if (slm_control (instance, connection, SLMC_SETHOLD, holdval) == SLM_OK)
		printf ("Setting hold time to %ld seconds.\n", holdval);
	else
		printf ("SETHOLD failed\n");
}

void
gethold()
{
	long holdval;

	if (slm_control (instance, connection, SLMC_GETHOLD, (long)&holdval) == SLM_OK)
		printf ("Hold time currently set to %ld seconds.\n", holdval);
	else
		printf ("GETHOLD failed\n");
}

#ifdef ANSI_C
void check_arg ( int opt, char *arg, char *msg ) {
#else
void
check_arg(opt, arg, msg)
char *arg, *msg;
{
#endif
	if (arg == NULL || arg[0] == '\0' || arg[0] == '-')
	{
		printf ("%s missing after -%c\n", msg, opt);
		exit (1);
	}
}

show_elmd_release()
{
	char elmd_release[64];
	int x;
	x = slm_control (instance, connection, SLMC_SLMD_RELEASE, (long) elmd_release);
	if (x == SLM_OK)
		printf ("    SLMD release=\"%s\"\n", elmd_release);
}

/*
 ***************************************************************************
 * MAIN
 ***************************************************************************
 */
#ifdef ANSI_C
void main ( int argc, char **argv ) {
#else 
void
main(argc, argv)
int argc;
char **argv;
{
#endif
	extern char *optarg;		/* in getopt() */
	extern int optind;		/* in getopt() */
	long cl,ret;
	int c, constraint_num = 0;
	char *feature = DEFAULT_FEATURE;
	char *user = NULL;
	long holdtime = -1L;
	unsigned long getoption = SLM_GETLIC;
	int SARusefeature = TRUE;
	int non_blocking_io = FALSE;
	long meter = 0;
	long ntokens = 1;	/* Number of tokens: default 1 */
#if defined(NETWARE) && !defined(_WINDOWS)
	/* UNLICENSED CODE REMOVED */
#endif		

	progname = argv[0];
	pid = getpid();

	if ((ret = slm_startapi (&instance)) != SLM_OK) {
		if (ret == SLM_CANT_OPEN || ret == SLM_CANT_READ) {
			printf ("Could not read salt information.\n");
			printf ("This program should be customized using elmcust, the binary customization utility.\n");
		} else {
			printf ("Initialization of library failed.\n");
		}
		exit (1);
	}

	while ((c = getopt(argc, argv, "f:h:b:qsFit:u:e:m:nc:")) != EOF)
	{
	    switch (c)
	    {
		case 'c':
			constraint_num = atoi(optarg);
			break;
		case 'n':
			non_blocking_io = TRUE;
			break;
		case 'm':
			check_arg (c, optarg, "Number");
			meter = atol (optarg);
			break;
		case 'e':
			check_arg (c, optarg, "Directory name");
			Keydir = optarg;
			break;
		case 'u':
			check_arg (c, optarg, "User name");
			user = optarg;
			break;
		case 't':
			check_arg (c, optarg, "Number");
			ntokens = atol (optarg);
			getoption |= SLM_COUNT;
			break;
		case 'i':
			printf ("CLIENT Release %s [VID %lx]\n",
						slm_getrelease(instance), slm_vid(instance));
			exit (0);
			break;
		case 'q':
			getoption |= SLM_QUEUE;
			break;
		case 's':
			getoption |= SLM_SHARED;
			break;
		case 'b':
			check_arg (c, optarg, "Number");
			beat = atoi (optarg);
			break;
		case 'f':
			check_arg (c, optarg, "Feature name");
			feature = optarg;
			break;
		case 'F':
			SARusefeature = FALSE;
			break;
		case 'h':
			check_arg (c, optarg, "Number");
			holdtime = atol (optarg);
			break;
		case '?':
	printf("Usage: %s [ -c# -m # -e keydir -f name -h # -b # -q -s -t # -F -i -u name ]\n",
				progname);
			exit (1);
	    }
	}

	if (non_blocking_io)
	{
		printf ("Using NON-BLOCKING I/O instead of SELECT.\n");
		slm_control (instance, SLM_NOCONNECTION, SLMC_HEARTBEAT_IO, SLM_USE_NON_BLOCKING_IO);
	}

	do_init(SARusefeature ? feature:(char *)0);	/* Initialization */

	if (user)
		slm_setuser (instance, connection, user, (char *)NULL, (char *)NULL);

	show_elmd_release();

	/* 
	 * Set a default minimum hold time for this process.
	 */
	if (holdtime > 0)
		sethold (holdtime);

	do_getlicense (feature, ntokens, getoption, constraint_num);/* Get a license */

	/*
	 * For demonstration purposes, we've implemented a hold time 
	 * of 0 here to mean turn of any holding, even if holding 
	 * is specified in the resource file.
	 */
	if (holdtime == 0)
		sethold (0L);

#if defined(NETWARE) && !defined(_WINDOWS)
	/* UNLICENSED CODE REMOVED */
#else
	slm_heartbeat (instance, connection, (long )beat, Exception);	/* Set up the heartbeat */
#endif	

	/*
	 * Here's where an application does it's normal stuff.
	 * We just loop forever.
	 * (We also exit if a keyboard interrupt or TERM signal is received.)
	 */

	IsRunning = TRUE;
	while (IsRunning) {
		printf ("[%ld] PID %lu is running.\n", slm_clientid(instance, connection), pid);
#if defined (WINDOWS_31) 
	/* UNLICENSED CODE REMOVED */
#else           
		slu_sleep (5);
#endif
#if defined(NETWARE) && !defined(_WINDOWS)
	/* UNLICENSED CODE REMOVED */
#endif
		if (meter)
		{
			if (meter > ntokens)
printf("    Attempting to meter %ld token%s with %ld left. (This should fail.)\n",
				meter, plural(meter), ntokens);
			printf ("    Decrementing license %ld tokens: ", meter);
			fflush (stdout);
			cl = slm_meter (instance, connection,
				feature, meter, 0L);
			if (cl)
			    printf ("%s\n", slm_message (instance, 0, 0, cl));
			else
			{
			    printf ("Done\n");
			    ntokens -= meter;
			}
			showinfo (feature);
		}
	}

	slm_disconnect (instance, connection);
            
	printf ("PID %lu exiting.\n", pid);

	slm_endapi (&instance);

	exit (0);
}
